#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Appier Framework
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Appier Framework.
#
# Hive Appier Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Appier Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Appier Framework. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import appier

from . import base


class DiagRequest(base.DiagBase):
    address = appier.field(index="all", immutable=True)

    url = appier.field(index="all", immutable=True)

    method = appier.field(index="all", immutable=True)

    path = appier.field(index="all", immutable=True)

    query = appier.field(index="all", immutable=True)

    code = appier.field(type=int, index="all", immutable=True, description="Status")

    protocol = appier.field(index="all", immutable=True)

    content_type = appier.field(index="all", immutable=True)

    data = appier.field(immutable=True)

    duration = appier.field(type=float, index="all", immutable=True)

    in_length = appier.field(type=int, index="all", immutable=True)

    out_length = appier.field(type=int, index="all", immutable=True)

    browser = appier.field(index="all", immutable=True)

    headers = appier.field(type=dict, immutable=True)

    browser_info = appier.field(type=dict, immutable=True)

    meta_info = appier.field(type=dict, immutable=True)

    geo_info = appier.field(type=dict, immutable=True)
